<?php
class Sipgate
{
	var $username;
	var $password;
	var $sip_id;

	// Init.
	
		function Sipgate($username, $password, $sip_id = false)
		{
			$this->username = $username;
			$this->password = $password;
			$this->sip_id = $sip_id;
		}

	// System methods.

		// Request a list of the RPC methods offered by the server with each entry being the name of a method stored as string value.
		// Returns the response as an array.
		function list_methods()
		{
			$result = $this->_request('system.listMethods');
			return $result['listMethods'];
		}

		// Request a human readable description of the given method containing information about purpose and usage.
		// Returns the response as an array.
		function method_help($method)
		{
			return $this->_request('system.methodHelp', array('MethodName' => $method));
		}

		// Request the signature of the given RPC method. 
		// The server will return a list of signatures valid for the specified method. 
		// The signature is defined as the promised return value?s type followed by the types of any arguments. 
		// A method MAY be called in more than one way, which is what the purpose of defining signatures is meant to distinguish.
		// Returns the response as an array.
		function method_signature($method)
		{
			return $this->_request('system.methodSignature', array('MethodName' => $method));
		}

		// The system.serverInfo method provides specific information on the server software.
		// Returns the response as an array.
		function server_info()
		{
			return $this->_request('system.serverInfo');
		}

	// Samurai functions.

		// Request an account statement for a specified period.
		// Returns the full response as an array.
		function account_statement($time_from = 0, $time_to = false)
		{
			if (!$time_to) $time_to = time();

			$args = array('PeriodStart' => date('c', $time_from), 'PeriodEnd' => date('c', $time_to));
			return $this->_request('samurai.AccountStatementGet', $args);
		}
		
		function event_list($time_from = 0, $time_to = false, $labels = array(), $type = array(), $limit = 0, $offset = 0, $events = array(), $start_event = false)
		{
			if (!$time_to) $time_to = time();

			$args = array('PeriodStart' => date('c', $time_from), 'PeriodEnd' => date('c', $time_to), 'Labels' => $labels, 'TOS' => $type, 'Limit' => $limit, 'Offset' => $offset, 'EventIDs' => $events,   'IncrementBaseID' => $start_event);
			return $this->_request('samurai.EventListGet', $args);
		}
		
		function event_summary($labels = array(), $type = array())
		{
			$args = array('LabelName' => $labels, 'TOS' => $type);
			return $this->_request('samurai.EventSummaryGet', $args);
		}
		
		function itemized_entries($uris = array(), $time_from = 0, $time_to = false)
		{
			if (!$time_to) $time_to = time();
			
			$args = array('PeriodStart' => date('c', $time_from), 'PeriodEnd' => date('c', $time_to));
			xmlrpc_set_type($args['PeriodStart'], 'datetime');
			xmlrpc_set_type($args['PeriodEnd'], 'datetime');
			return $this->_request('samurai.ItemizedEntriesGet', $args);
		}
		
		
		

		// The client SHOULD identify itself to the server including version and vendor information to allow for better support in cases of problems.
		// Returns true on success or false on failure.
		function identify($client_vendor, $client_name, $client_version = '1.0')
		{
			$args = array('ClientName' => $client_name, 'ClientVersion' => $client_version, 'ClientVendor' => $client_vendor);
			return $this->_request('samurai.ClientIdentify', $args);
		}

		// Request the list of SIP URIs belonging to the user's account.
		// Returns array of SIP URIs on success, false on failure.
		function uris()
		{
			if ($response = $this->_request('samurai.OwnUriListGet')) return $response['OwnUriList'];
			else return false;
		}

		// Request the current balance of the users account in pence
		// Returns balance on success or false on failure (remember to === false in case of zero balance).
		function balance()
		{
			if ($response = $this->_request('samurai.BalanceGet')) return $response['CurrentBalance']['TotalIncludingVat']; // Round to pence.
			else return false;
		}
		
		function format_number($number)
		{
			if($number) {
				preg_match_all('/\d/',$number, $matches);
				$clean = join('',$matches[0]);
				return $number = "49".preg_replace(array('/^0*/','/^49/','/^0*/'), '', $clean);
			}
		}

		// Click-to-dial.
		// $number must have no spaces and include an international calling code, but no IDD or NDD prefixes (0, 00, +, etc).
		// Returns SessionID on success, false on failure.
		function call($number, $time = false)
		{
			$number = $this->format_number($number);
			
			if (!$this->sip_id)
			{
				trigger_error('SIP-ID is required for click-to-dial.', E_USER_ERROR);
				return false;
			}
			
			$args = array('LocalUri' => 'sip:' . $this->sip_id . '@sipgate.de', 'RemoteUri' => 'sip:' . $number . '@sipgate.net', 'TOS' => 'voice', 'Content' => '');
			if ($time) $args['Schedule'] = date('c', $time);
			
			if ($response = $this->_request('samurai.SessionInitiate', $args)) return $response['SessionID'];
			else return false;
		}

		// Hang up after a click-to-dial.
		// $session_id must be an open session.
		// Returns true on success, false on failure.
		function call_end($session_id)
		{
			$args = array('SessionID' => $session_id);

			if ($response = $this->_request('samurai.SessionClose', $args)) return true;
			else return false;
		}

		// Get status of a session.
		// $session_id must be a valid session.
		// Returns human-readable status on success, false on failure (or session not found, ie call ended).
		function call_status($session_id)
		{
			$args = array('SessionID' => $session_id);

			if ($response = $this->_request('samurai.SessionStatusGet', $args)) return $response['SessionStatus'];
			else return false;
		}

		// Send a text message to a phone number.
		// $country must be a two-letter ISO code.
		// $number must have no spaces and include an international calling code, but no IDD or NDD prefixes (0, 00, +, etc).
		//  It can also be an array of numbers to send the text to multiple phones.
		// Returns true on success, false on failure.
		function text($number, $message, $sender = false, $time = false)
		{
			$number = $this->format_number($number);
			$sender = $this->format_number($sender);
			if (is_array($number))
			{
				$args = array('RemoteUri' => array(), 'TOS' => 'text', 'Content' => utf8_decode(str_replace("\r\n", "\\n", $message)));

				foreach ($number as $t_number) $args['RemoteUri'][] = 'sip:' . $t_number . '@sipgate.net';
				if ($sender) $args['LocalUri'] = 'sip:'.$sender.'@sipgate.de';
				if ($time) $args['Schedule'] = date('c', $time);

				if ($response = $this->_request('samurai.SessionInitiateMulti', $args)) return $response['SessionID'];
				else return false;
			}
			else
			{
				$args = array('RemoteUri' => 'sip:' . $number . '@sipgate.net', 'TOS' => 'text', 'Content' => utf8_decode(str_replace("\r\n", "\\n", $message)));
				if ($sender) $args['LocalUri'] = 'sip:'.$sender.'@sipgate.de';
				if ($time) $args['Schedule'] = date('c', $time);

				if ($response = $this->_request('samurai.SessionInitiate', $args)) return $response['SessionID'];
				else return false;
			}
		}

	// Private functions.

		// Do a request and throw an error on failure.
		// Returns true on success and false on failure.
		private function _request($method, $args = array())
		{
			$url = 'https://api.sipgate.net/RPC2';
			//echo date('c', time());
			$request = xmlrpc_encode_request($method, $args);
			//echo  base64_encode($this->username . ':' . $this->password);
			//echo $request;
			$context = stream_context_create(array('http' => array(
				'method' => 'POST',
				'header' => array(
					"Content-Type: text/xml", 
					"Authorization: Basic " . base64_encode($this->username . ':' . $this->password)
				),
				'content' => $request
			)));

			$file = file_get_contents($url, false, $context);
			
			//echo $file;
			$response = xmlrpc_decode($file);

			if ($response['StatusCode'] == 200) return ($response);
			else
			{
				if ($response['faultCode']) trigger_error($response['faultCode'] . ': ' . $response['faultString'], E_USER_ERROR);
				elseif ($response['StatusCode']) trigger_error($response['faultString'], E_USER_ERROR);
				else trigger_error('Sipgate error.', 'Unknown error in Sipgate response.', E_USER_ERROR);
				return false;
			}
		}
}
?>